<?php declare(strict_types=1);

/*
 *     This file is part of sikofitt/user-agent
 *
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\UserAgent;

use Sikofitt\Retrorsum\Finder;

class UserAgent
{
    protected static $format = 'PHP/%d.%d%s-%d (%s; %s %s) PHP/%s%s; %s (%s)';
    protected static $params = [];

    /**
     * @return string
     */
    final public static function create(): string
    {
        if (empty(static::$params)) {
            $params = self::getDefaultParams();
            self::setParams(...$params);
        }

        return sprintf(static::$format, ...static::$params);
    }

    private static function setParams(...$params): void
    {
        static::$params = $params;
    }

    /**
     * @return array
     */
    private static function getDefaultParams(): array
    {
        $finder = new Finder('vendor');

        $packageString = '';
        $packageUrl = '';

        if ($finder->isFound()) {
            $json = file_get_contents($finder->getPath() . '/composer.json');
            $composer = json_decode($json, true);
            $packageString = $composer['name'];
            $packageUrl = sprintf('https://packagist.org/%s', $packageString);
        }

        $zts = static::getZts();

        return [
          PHP_MAJOR_VERSION,
          PHP_MINOR_VERSION,
          $zts,
          PHP_VERSION_ID,
          $packageString,
          PHP_OS_FAMILY,
          static::getArch(),
          PHP_VERSION,
          $zts,
          static::getGuzzleVersionString(),
          $packageUrl,
        ];
    }

    /**
     * @return string
     */
    private static function getZts(): string
    {
        return (bool)PHP_ZTS ? '-zts' : '';
    }

    /**
     * @return string
     */
    private static function getGuzzleVersionString(): string
    {
        /** @noinspection ClassConstantCanBeUsedInspection */
        if (class_exists('GuzzleHttp\Client')) {
            /** @noinspection PhpUnnecessaryFullyQualifiedNameInspection */
            $guzzleClient = new \GuzzleHttp\Client();
            $guzzleHeaders = $guzzleClient->getConfig('headers');
            $guzzleUserAgent = $guzzleHeaders['User-Agent'];
            return trim(explode('php', strtolower($guzzleUserAgent))[0]);
        }

        return '';
    }

    /**
     * @return string
     */
    private static function getArch(): string
    {
        return 9223372036854775807 === PHP_INT_MAX ? 'x86_64' : 'i386';
    }
}
